#include <iostream>
using std::cout;


// a primitive "smart" array
template <class T>
class vec {
public:
   vec();
   ~vec();
   void push_back(const T&);  // put an element onto the end of the array
   void display();  // display all elements of array to cout
private:
   int capacity;    // how much space is currently available
   int size;        // how much space is currently used
   T *contents;     // pointer to array containing data
};

template <class T>
void vec<T>::display()
{
    int i;
    // loop through array and display one-by-one
    for (i=0; i < size; i++)
        cout << contents[i] << " "; 
}

// constructor - start with enough room for 1 element
template <class T>
vec<T>::vec()
:capacity(1), size(0)
{
    contents = new T[1];
}

template <class T>
vec<T>::~vec()
{
    // release dynamically allocated memory, reset all variables
    delete [] contents;
    contents=0;
    size=0;
    capacity=0;
}

template <class T>
void vec<T>::push_back(const T&newval)
{
    T *newarr;
    int i;
    if (size==capacity) {   // not enough room!
        newarr =  new T[capacity*2];   // make a new array
        for (i=0; i<capacity; i++)     // copy it
            newarr[i] = contents[i];
        capacity *=2;                  // double the capacity
        delete [] contents;            // release old array
        contents = newarr;             // point to newly allocated larger array
    }
    contents[size] = newval;           // store the data at the end of the array
    size++;                            // array is one element longer now
}


int main()
{
    int i;
    vec <int> myvec;
    for (i=0; i < 10; i++)
        myvec.push_back(i);    
    myvec.display();
    cout << endl;
}
